// Simple Paint
// Composed for the edification of students by Brian Moriarty
// Released under GLPL-3.0

// The following comment line is for JSLint. Don't remove it!

/*global PS */

var PAINT = {
	WIDTH: 16, // width of grid
	HEIGHT: 17, // height of grid (one extra row for palette)
	PALETTE_ROW: 16, // row occupied by palette
	WHITE: 8, // x-position of white in palette
	ERASE_X: 15, // x-position of X in palette
	
	// the palette colors, scientifically chosen!
	
	COLORS: [
		0xFF0000, 0xFF8000, 0xFFFF00, 0x00C000, 0x00FFFF,
		0x4040FF, 0x8040FF, 0xFF00FF, 0xFFFFFF, 0xC0C0C0,
		0xA0A0A0, 0x808080, 0x606060, 0x404040, 0x000000 ],	
		
	Current: 8, // x-pos of current palette selection
	Color: PS.COLOR_WHITE, // color of current palette selection
	UnderColor: PS.COLOR_WHITE, // color of bead under the brush
	Dragging: false, // true if dragging brush
	Prompt: false // true if instructions displayed
};

PS.Init = function ( options )
{
	"use strict";
	var i, lastx, lasty, color;

	PS.GridSize( PAINT.WIDTH, PAINT.HEIGHT );
	PS.BeadFlash( PS.ALL, PS.ALL, false ); // disable all flashing
	PS.BeadBorderWidth( PS.ALL, PS.ALL, 0 ); // disable all borders
	
	// preload sounds
	
	PS.AudioLoad( "fx_click" );
	PS.AudioLoad( "fx_pop" );
	
	// Draw palette
	
	lastx = PAINT.WIDTH - 1;
	lasty = PAINT.PALETTE_ROW; // faster if saved in local var
	for ( i = 0; i < lastx; i += 1 )
	{
		color = PAINT.COLORS[i];
		PS.BeadColor( i, lasty, color ); // set visible color
		PS.BeadData ( i, lasty, color ); // also store color as bead data
		PS.BeadAudio ( i, lasty, "fx_click" ); // click when selecting a color
		PS.BeadFunction ( i, lasty, PAINT.Select ); // call PAINT.Select when clicked
		
		// set border color according to position
		
		if ( i < 12 )
		{
			color = 0x000000; // black for light colors
		}
		else
		{
			color = 0xC0C0C0; // light gray for dark colors
		}
		PS.BeadBorderColor( i, lasty, color );
	}
	
	// set up reset button
	
	PAINT.ERASE_X = lastx; // remember the x-position
	PS.BeadGlyphColor( lastx, lasty, PS.COLOR_BLACK );
	PS.BeadGlyph( lastx, lasty, "X" );
	PS.BeadAudio ( lastx, lasty, "fx_pop" ); // pop on reset
	PS.BeadFunction ( lastx, lasty, PAINT.Reset ); // call PAINT.Reset when clicked
	
	// start with white selected
	
	PS.BeadBorderWidth(PAINT.WHITE, PAINT.PALETTE_ROW, 2);	
	PAINT.Current = PAINT.WHITE;
	PAINT.Color = PS.COLOR_WHITE;	
	
	PAINT.Reset();
	
	PS.StatusText("Simple Paint");
};

// Called when a palette bead is clicked

PAINT.Select = function ( x, y, data )
{
	"use strict";
	var color;
	
	// activate border if changing selection
	
	if ( x !== PAINT.Current )
	{
		PS.BeadBorderWidth(PAINT.Current, PAINT.HEIGHT - 1, 0); // turn off previous border
		PS.BeadBorderWidth( x, y, 2 );
		PAINT.Current = x;
		PAINT.Color = data; // set current color from color stored in bead data
	}		
};

// Clear the canvas

PAINT.Reset = function ( x, y, data )
{
	"use strict";
	var i, lasty;
	
	PAINT.Dragging = false;
	PAINT.UnderColor = PS.COLOR_WHITE;
	for ( i = 0; i < PAINT.PALETTE_ROW; i += 1 )
	{
		PS.BeadColor( PS.ALL, i, PS.COLOR_WHITE );
	}	
};

// Called when mouse button is pressed

PS.Click = function (x, y, data, options)
{
	"use strict";
	
	if ( y < PAINT.PALETTE_ROW )
	{
		PAINT.Dragging = true;
		PAINT.UnderColor = PAINT.Color;
		PS.BeadColor( x, y, PAINT.Color );
	}
};

// Called when mouse button is released

PS.Release = function (x, y, data, options)
{
	"use strict";

	PAINT.Dragging = false;
};

// Called when mouse enters a bead

PS.Enter = function (x, y, data, options)
{
	"use strict";
	
	if ( y < PAINT.PALETTE_ROW )
	{
		PAINT.UnderColor = PS.BeadColor( x, y );
		PS.BeadColor( x, y, PAINT.Color );
		if ( PAINT.Dragging )
		{
			PAINT.UnderColor = PAINT.Color;
		}
	}
	else
	{
		PAINT.Dragging = false; // stop dragging if over palette
		if ( x === PAINT.ERASE_X )
		{
			PAINT.Prompt = false;
			PS.StatusText("Click X to erase painting");
		}
	}
};

// Called when mouse leaves a bead

PS.Leave = function (x, y, data, options)
{
	"use strict";

	// Show instructions when mouse is first moved 
	
	if ( !PAINT.Prompt )
	{
		PAINT.Prompt = true;
		PS.StatusText("Click to select colors, click/drag to paint");
	}
	
	if ( y < PAINT.PALETTE_ROW )
	{
		PS.BeadColor( x, y, PAINT.UnderColor );
	}
};

// These functions aren't used
// They're here to prevent error messages on startup

PS.KeyDown = function (key, shift, ctrl, options)
{
	"use strict";
};

PS.KeyUp = function (key, shift, ctrl, options)
{
	"use strict";
};

PS.Wheel = function (delta, options)
{
	"use strict";
};

PS.Tick = function (options)
{
	"use strict";
};
